Unit pump;


interface

const     P_contrast = $1FD;
          P_setup    = $1FC;
          P_lcd      = $1FF;
          P_tout     = 10000;
          D_delay    = 60;

          Release  = 1;
          RelDate  = '13.05.2000';
          PCFGver  = 3;
          OLMver   = 3;
          PUMPVer  = 03;
          GLCDVer  = 01;



var       lpt : Word; { Address of printerport default is $378 }
          readserial : boolean;
          P_timeout  : boolean;

          LCDbuf     : Array[1..4] of string;


procedure Seteeprom(Adr : Word; Data : Byte); { write to PUMP EEPROM }
function  Geteeprom(Adr : Word) : Byte;       { read from PUMP EEPROM }

procedure writew(reg,dat : Byte);
procedure writex(reg,dat : Byte);             { write to PUMP }
function  readx(reg : Byte) : Byte;           { read from PUMP }

Function  GetVersion : Byte;

Function  GetPUMPcfg : Byte;                  { Get System Configuration }
Procedure SetPUMPcfg( b : byte);              { Set System Configuration }

Function  GetPUMPlcd : Byte;                  { Get LCD Type }
Procedure SetPUMPlcd( b : Byte);              { Set LCD Type }

Function  GetContrast : Byte;                 { Get the contrast }

Function  GetBacklight : Byte;                { Get the backlight intensity }

procedure GetDispDim;        { get the actual configured LCD dimensions }
procedure UpdateDisp;        { flush the buffer to the lcd screen }

procedure ResetPUMP;                          { Resets the PUMP }

procedure ExitTimeout( s : string);


function  version : string;

implementation

uses DOSE50;

var l4x40 : Boolean;  { true if a 4x40 LCD is configured }
    xmax  : Byte;     { width of the LCD}
    ymax  : Byte;     { height of the LCD }


procedure put_port(port : Word; dat : Byte);
begin
  asm
    mov dx,(port)
    mov al,(dat)
    out dx,al
  end;
end;


function get_port(port : Word) : Byte;
var help : Byte;
begin
  asm
    mov dx,(port)
    in  al,dx
    in  al,dx
    in  al,dx
    mov (help),al
  end;
  get_port:=help;
end;


procedure writex(reg,dat : Byte);
const strg : Array[0..5] of Byte = ($0a,$02,$08,$00,$0c,$04);
var   t    : word;
begin
  put_port(lpt+2,strg[reg]);
  put_port(lpt,dat);
  put_port(lpt+2,strg[reg]+1);
  t := P_tout;
  repeat
  dec( t);
  until (get_port(lpt+1) and $80 <> 0) or (t=0);
  if t=0 then ExitTimeout( 'timout while setup in writex('+hex( reg)+','+hex( dat)+')');
  put_port(lpt+2,strg[reg]);
  t := P_tout;
  repeat
    dec( t);
  until (get_port(lpt+1) and $80 = 0) or (t=0);
  if t=0 then ExitTimeout( 'timeout while release in writex('+hex( reg)+','+hex( dat)+')');
end;


function readx(reg : Byte) : Byte;
const strg : Array[0..3] of Byte = ($2e,$26,$2c,$24);
var i,x,p : Byte;
    t     : word;
begin
  if readserial then begin                  { SERIAL MODE }
    put_port(lpt+2,strg[reg]);
    put_port(lpt+2,strg[reg]+1);
    t := P_tout;
    repeat
      p:=get_port(lpt+1);
      dec(t);
    until (p and $80 <> 0) or (t=0);
    if t=0 then ExitTimeout( 'timeout while setup in serial readx('+hex( reg)+')');
    if p and $08 <> 0 then x:=1
    else x:=0;
    for i:=1 to 7 do
      begin
        put_port(lpt+2,strg[reg]+1-4);
        repeat
          p:=get_port(lpt+1);
        until p and $80 = 0;
        put_port(lpt+2,strg[reg]+1);
        t := P_tout;
        repeat
          p:=get_port(lpt+1);
          dec(t);
        until (p and $80 <> 0) or (t=0);
        if t=0 then ExitTimeout( 'timeout while reading in serial readx('+hex( reg)+')');
        if p and $08 <> 0 then x:=x + (1 shl i);
      end;
    put_port(lpt+2,strg[reg]);
    t := P_tout;
    repeat
      dec(t);
    until (get_port(lpt+1) and $80 = 0) or (t=0);
    if t=0 then ExitTimeout( 'timeout in parallel readx('+hex( reg)+')');
    readx:=x;
  end
  else begin                                { PRALLEL MODE }
    put_port( lpt+2, strg[reg]);
    put_port( lpt+2, strg[reg]+1);
    repeat
    until get_port( lpt+1) and $80 <> 0;
    readx:=get_port( lpt);
    put_port( lpt+2, strg[reg]);
    repeat
    until get_port( lpt+1) and $80 = 0;
  end;
end;


procedure writew(reg,dat : Byte);
var ii : Word;
begin
  { Warten bis Display-Busy ende }
  ii:=0;
  if reg>1 then
    repeat
      ii:=ii+1;
    until (readx(2) and $80=0) or (ii>1000);
  writex(reg,dat);
end; {writew}


procedure Seteeprom(Adr : Word; Data : Byte);
begin
  if Adr<512 then begin
    writex(4,Adr and $ff);
    writex(1,$23);
    writex(4,Adr shr 8);
    writex(1,$24);
    writex(4,Data);
    writex(1,$25);
  end;
end;


function Geteeprom(Adr : Word) : Byte;
begin
  writex(1,$15);
  if Adr<512 then begin
    writex(4,Adr and $ff);
    writex(1,$23);
    writex(4,Adr shr 8);
    writex(1,$24);
    Geteeprom:=readx(1);
  end;
end;


Function GetVersion : Byte;
begin
 writex( $01, $1d);
 GetVersion := readx( $01);
end;



Function GetPUMPcfg : Byte;
begin
  GetPUMPcfg := GetEEPROM( P_setup);
end;

Procedure SetPUMPcfg( b : byte);
begin
  SetEEPROM( P_setup, b);
end;



Function GetPUMPlcd : Byte;
begin
  GetPUMPlcd := GetEEPROM( P_lcd);
end;

Procedure SetPUMPlcd( b : Byte);
begin
  SetEEPROM( P_lcd, b);
end;



Function GetContrast : Byte;
begin
  writex( $01, $18);
  GetContrast := readx( $01);
end;


Function GetBacklight : Byte;
begin
  writex( $01, $19);
  GetBacklight := readx( $01);
end;


procedure GetDispDim;
begin
  l4x40 := (GetPUMPcfg and $80)=0;
  if l4x40 then begin
    xmax := 40;
    ymax := 4; end
  else
  case GetPUMPlcd of
    01   : begin xmax := 16; ymax := 2; end;
    02   : begin xmax := 20; ymax := 2; end;
    03   : begin xmax := 16; ymax := 4; end;
    else   begin xmax := 20; ymax := 4; end;
  end;
end;


procedure UpdateDisp;
var x, y  : Byte;
begin
  for y := 1 to ymax do begin
    if l4x40 then
      case y of
        1..2 : writex( 1, $40);
        3..4 : writex( 1, $41);
      end;
    for x := 1 to xmax do begin
      if x <= ord( LCDbuf[y][0]) then Writex( $3, ord( LCDbuf[y][x]))
                                 else Writex( $3, 32);
    end;
  end;
end;


Procedure ResetPUMP;
begin
  writex( $01, $04);
end;


function version : string;
begin
  version := dby( Release)+'.'+dby( PCFGver)+'.'+dby(OLMver)+'.'+dby( PUMPver);
end;


procedure ExitTimeout( s : string);
begin
  Writeln( s);
  Halt( 1);
end;

begin
  lpt := $378;
  readserial := true;
  xmax  := 20;
  ymax  := 4;
  l4x40 := false;
end.